def initialize(context, product):
    misc = product.module.misc_
    from OFS.ObjectManager import ObjectManager
    ObjectManager.manage = misc['manage']
    from App.Management import Tabs
    Tabs.manage_tabs = misc['manage_tabs']

    product.module._m = __FactoryDispatcher__.__dict__
    __FactoryDispatcher__.bookmark_list_pt = misc['bookmark_list']
    __FactoryDispatcher__.bookmarks_edit_pt = misc['bookmarks_edit']
    __FactoryDispatcher__.bookmarks_file_pt = misc['bookmarks_file']

from App.FactoryDispatcher import FactoryDispatcher
from Acquisition import aq_parent
class __FactoryDispatcher__(FactoryDispatcher):
    "Factory Dispatcher for ZMIBookmarks"

    def bookmark_list(self, REQUEST, add=None, download=None):
        "List Bookmarks, optionally as a text download"
        sess = REQUEST.SESSION

        marks = sess.get('zmi_bookmarks')
        if add:
            path = filter(None, add.split('/'))
            mark = ('/'.join(path), path[-1])

            if marks:
                if mark not in marks:
                    marks.append(mark)
            else:
                sess['zmi_bookmarks'] = marks = [mark]
            REQUEST.RESPONSE.redirect(REQUEST['URL'])

        if download:
            REQUEST.RESPONSE.setHeader('Content-type', 'text/plain')
            return self.marks_to_text(marks)
        
        return self.bookmark_list_pt(marks=marks)

    def marks_to_text(self, marks):
        if not marks:
             return ''
        text = []
        for mark in marks:
            if mark[0] == mark[1] or mark[0].endswith('/' + mark[1]):
                text.append(mark[0] + '\n')
            else:
                text.append('%s # %s\n' % (mark[0], mark[1]))
        return ''.join(text)

    def marks_from_text(self, text):
        marks = []
        for line in text.split('\n'):
            parts = line.split('#', 1)
            path = filter(None, parts[0].strip().split('/'))
            if not path:
                continue
            if len(parts) == 2:
               name = parts[1].strip()
            else:
               name = path[-1]
            marks.append(('/'.join(path), name))
        return marks
    
    def bookmarks_edit(self, REQUEST, new_marks=None, arrange=None, file=None):
        "Read Bookmarks from form data or file"
        sess = REQUEST.SESSION

        if new_marks:
            marks = []
            for mark in new_marks:
                if mark.name:
                    path = filter(None, mark.path.split('/'))
                    marks.append(('/'.join(path), mark.name))
                else:
                    marks.append(None)
            if arrange and hasattr(arrange, 'from'):
                k = arrange.keys()
                if len(k) == 2:
                    m = marks.pop(int(getattr(arrange, 'from')))
                    marks.insert(int(max(k)[3:]), m)

            sess['zmi_bookmarks'] = filter(None, marks)

        elif getattr(file, 'filename', 0):
            sess['zmi_bookmarks'] = self.marks_from_text(file.read())

        return self.bookmarks_edit_pt()

    def bookmarks_file(self, REQUEST, act=None):
        "Load/Save Bookmarks using a File object"
        if act is not None:
            ob = aq_parent(aq_parent(self))
            goto = REQUEST['URL']
            sess = REQUEST.SESSION
            if act == 'load':
                sess['zmi_bookmarks'] = self.marks_from_text(str(ob))
                goto = REQUEST['URL1'] + '/bookmarks_edit'
            elif act == 'save':
                marks = sess['zmi_bookmarks']
                ob.update_data(self.marks_to_text(marks))
                goto = REQUEST['URL3'] + '/manage_workspace'
            REQUEST.RESPONSE.redirect(goto)
            return
        return self.bookmarks_file_pt()
        
